//Inputs and outputs 
int sensor_in = A0;
int delay_in = A1;
int brightness = A2;
int relay_out = 3;

//Variables for the code
int detected = 0;
int firing_pulse_delay=0;
int last_CH1_state = 0;
bool relay_activated = false;
unsigned long delay_time = 5000;    //Initial delay time = 5 seconds
unsigned long previousMillis = 0; 


void setup() {
  pinMode(sensor_in,INPUT);
  pinMode(delay_in,INPUT);
  pinMode(brightness,INPUT);
  pinMode(relay_out,OUTPUT);
  /*
   * Port registers allow for lower-level and faster manipulation of the i/o pins of the microcontroller on an Arduino board. 
   * The chips used on the Arduino board (the ATmega8 and ATmega168) have three ports:
     -B (digital pin 8 to 13)
     -C (analog input pins)
     -D (digital pins 0 to 7)   
  //All Arduino (Atmega) digital pins are inputs when you begin...  */  
   
  PCICR |= (1 << PCIE0);    //enable PCMSK0 scan                                                 
  PCMSK0 |= (1 << PCINT0);  //Set pin D8 trigger an interrupt on state change. Input from optocoupler
  pinMode(3,OUTPUT);        //Define D3 as output for the TRIAC pulse

}

void loop() {
  delay_time = map(analogRead(delay_in),0,1024,5000,60000);     //Map delay from 5 to 60 seconds
  
  if(analogRead(sensor_in) > 500 && !relay_activated)
  {
    relay_activated = true;  
    previousMillis = millis(); 
  }

  if(analogRead(sensor_in) < 500)
  {
    unsigned long currentMillis = millis();
    if(currentMillis - previousMillis >= delay_time)
    {
      previousMillis += delay_time;
      relay_activated = false; 
    }
  }


  if(relay_activated)
  {
    firing_pulse_delay = map(analogRead(brightness),0,1024,7200,10);
    if (detected)
    {
      delayMicroseconds(firing_pulse_delay); //This delay controls the power
      digitalWrite(3,HIGH);
      delayMicroseconds(100);
      digitalWrite(3,LOW);
      detected=0;
    }
  }
  
  
      


   
  
}//end of void loop






//This is the interruption routine
//----------------------------------------------

ISR(PCINT0_vect){
  /////////////////////////////////////               //Input from optocoupler
  if(PINB & B00000001){                               //We make an AND with the pin state register, We verify if pin 8 is HIGH???
    if(last_CH1_state == 0){                          //If the last state was 0, then we have a state change...
      detected=1;                                    //We haev detected a state change!
    }
  }
  else if(last_CH1_state == 1){                       //If pin 8 is LOW and the last state was HIGH then we have a state change      
    detected=1;                                      //We haev detected a state change!
    last_CH1_state = 0;                               //Store the current state into the last state for the next loop
    }
}
